/* 
 * Global error handler for icinga-web
 * .
 */

Ext.ns("AppKit.errorHandler");
(function() {
	AppKit.BugTracker = new function() {
		var errorMsg = function(msg,file,line) {
			this.msg = 'No message available';
			this.file = 'No file available';
			this.line = 'No line available';
			this.stack = 'No stacktrace available';
			this.comment = "<span style='color:#ff0000'>No comment</span>";
			this.time = new Date().toLocaleString();
			try {
				this.msg = msg;
				this.file = file;
				this.line = line;

			//	this.stack = printStackTrace({e:msg,guess:false})
			} catch(e) {}
		};

		var errorReport = function() {	
			this.getHeader = function() {
				return  ";---------------------------------------------------\n"+
					";Icinga Interface Error Report	\n"+
					";---------------------------------------------------\n\n"+
					";Header definitions\n"+
					"CreationTime = '"+((new Date()).toLocaleString() || 'Unknown')+"'\n"+
					"URL = '"+(window.location.href || 'Unknown')+"'\n"+
					"Platform = '"+(navigator.platform || 'Unknown')+"'\n"+
					"User-Agent = '"+(navigator.userAgent || 'Unknown')+"'\n"+
					"\n";
			}
			
			this.text = ''
			this.send = function() {}
			this.show = function() {
				this.update();
				var id = Ext.id('errorProtocol');
				(new Ext.Window({
					title: _('Error report'),
					autoHeight:true,
					id: id,
					width:800,
					constrain:true,
					items: [{
						html: "<div style='height:300px;width:100%;overflow:scroll' class='icinga-codeBox'>"+this.text+"</div>"
					},{
						html: "<div style='width:100%;height:100%;font-size:12px;padding:5px;text-align:center'>If this bug is not already reported, feel free to report it at our <a href='https://dev.icinga.org/projects/icinga-web'>Bugtracker</a> (Don't forget to attach this report).</div>"
					}],
					buttons: [{
						text: _('Close'),
						iconCls: 'icinga-icon-close',
						handler: function() {
							Ext.getCmp(id).close();
						}
					}]
				})).show(document.body);
			}
			this.update = function() {
				this.buildText();
			}
			
			this.buildText = function() {
				this.text = this.getHeader();
				this.text += ";The following errors occured\n";
				var ctr = 1;
				Ext.each(occuredErrors, function(error) {
					var textMsg = "[Error "+(ctr++)+"]\n";
					textMsg += 
						"Message = '"+error.msg+"'\n"+
						"File = '"+error.file+"'\n"+
						"Line = '"+error.line+"'\n"+
						"Time = '"+error.time+"'\n"+
						"Comment = '"+error.comment+"'\n\n";
					this.text += textMsg;
					
				},this);
				this.text += ";EOF";
			}
			
			this.buildText();

		}

		var occuredErrors = [];
		var suspended = false;
		var showErrors = true;
		var handleError = function(msg,file,line) {
			var curError = new errorMsg(msg,file,line);
			occuredErrors.push(curError);
			if(showErrors) {
				updateErrorDisplay();
			}

		};
		var bugReportField = null;
		var updateErrorDisplay = function() {
			if(!bugReportField)
				setupErrorDisplay();
			else {
				bugReportField.setText(occuredErrors.length);
				Ext.getCmp('menu-navigation').doLayout();
			}
			
		}

		var setupErrorDisplay = function() {

			var elem = Ext.getCmp('menu-navigation');
			bugReportField = new Ext.Button({
				text: occuredErrors.length,
				iconCls: 'icinga-icon-bug',
				handler: AppKit.BugTracker.showErrorMessageInfoBox
			})
			elem.addItem(bugReportField);
			elem.doLayout();
			
		}

		window.onerror = handleError;

		return {
			clearErrors : function() {
				occuredErrors = [];
				updateErrorDisplay();
			},
			getErrors: function() {
				return occuredErrors;
			},
			setError: function(msg,file,line) {
				handleError(msg,file,line);
			},

			suspend: function() {
				window.onerror = function() {};
				suspended = true;
			},

			resume: function() {
				window.onerror = handleError;
				suspended = false;
			},

			isSuspended: function() {
				return suspended
			},

			setShowErrors: function(bool) {
				showErrors = bool;
			},

			showErrorMessageInfoBox: function() {
				var data = [];
				var i=0;
				Ext.each(occuredErrors,function(error) {
					data.push([i++,error.msg,error.file,error.line,error.time,error.comment]);
				})
				var dview = new Ext.DataView({
					store:new Ext.data.ArrayStore({
						fields: ['id','msg','file','line','time','comment'],
						idIndex: 0,
						data: data,
						autoDestroy: true
					}),
					tpl: new Ext.XTemplate(
						'<tpl for=".">',
							'<div qtip="Click to comment this bug" class="icinga-bugBox">',
								'<b>Message</b>: {msg}<br/>',
								'<b>File</b>: {file}<br/>',
								'<b>Line</b>: {line}<br/>',
								'<b>Occured</b>: {time}<br/>',
								'<b>Comment</b>: {comment}',
							'</div>',
						'</tpl>'),
					listeners: {
						click: function(dview,index,node,event) {
							var error = dview.getStore().getAt(index);
							Ext.Msg.prompt(
								_("Comment bug"),
								_("Please enter a comment for this bug. This could be"+
								  "<div style='background-color: #ffffff;padding:5px;-moz-border-radius:5px;-webkit-border-radius:5px'><ul style='list-style-type:circle'><li>What did you do when it occured?</li><li>Could you reproduce it? How?</li><li>Did you encounter any problems with the interface when the bug occured</li><li>If it happened after an update: Did the feature work in prior versions?</li></ul></div>"),
								function(btn,text) {
									if(btn != 'ok')
										return false;
									error.set('comment', encodeURI(text));
									dview.refresh();
									occuredErrors[index]["comment"] = encodeURI(text);
								},this,true
							);
						},
						scope:this
					},
					overClass:'xover',
					itemSelector:'div.icinga-bugBox'

				});
				var boxId = Ext.id('box_bug');
				var box = new Ext.Window({
					modal:true,
					height: 400,
					constrain:true,
					width:700,
					title: _('Bug report'),
					layout:'auto',
					id: boxId,
					items: [{
						padding:5,
						html:'<div class="icinga-icon-bug-32" style="padding-left:35px;padding-top:2px;height:32px;overflow:visible"><h2>'+_('Icinga bug report')+'</h2></div>'+
							'<br/>'+_('The following '+occuredErrors.length+' error(s) occured, sorry for that:')
					},{
						layout:'auto',
						xtype:'panel',
						collapsible:true,
						height:250,
						autoScroll:true,
						padding:5,
						items:dview
					}],

					buttons: [/*{
						text: _('Send report to admin'),
						iconCls: 'icinga-icon-application-form',
						handler: function() {new errorReport().send();},
						scope:this
					},*/{
						text: _('Create report for dev.icinga.org'),
						iconCls: 'icinga-icon-information',
						handler: function() {new errorReport().show();},
						scope:this
					},{
						text: _('Clear errors'),
						iconCls: 'icinga-icon-delete',
						handler: function() {
							AppKit.errorHandler.clearErrors()
							Ext.getCmp(boxId).close();
						},
						scope:this
					},{
						text: _('Close'),
						iconCls: 'icinga-icon-cancel',
						handler: function() {
							Ext.getCmp(boxId).close();
						}
					}]
				}).show(document.body);
			}
		}

	}
	AppKit.AjaxErrorHandler = new function() {
		
		var notifyBoxEnabled = true;
		var bugTrackerReportEnabled = true;
		// set user settings on startup
		

		var trackError = function(msg,src,line,isBug) {
			src = src || 'Unknown';
			line  = line || 'Unknown';
			if(notifyBoxEnabled)
				AppKit.notifyMessage(_('Request failed'),msg);
			if(isBug && bugTrackerReportEnabled)
				AppKit.BugTracker.setError(msg,src,line);
		}

		// Set up error handling for all automatic requests
		Ext.data.DataProxy.on('exception',function(proxy,type,action, options,response, arg) {
			handleError(response,proxy);
		});

		// Setup error handling for Ext.Ajax
		Ext.Ajax.on("requestException",function(conn,response,opts) {
			handleError(response,opts);
		})
		var handleError = function(response,proxy) {
			switch(response.status) {
				case 200:
					break;
				case 404:
					AppKit.AjaxErrorHandler.error_404(proxy.url);
					break;
				case 401:
					AppKit.AjaxErrorHandler.error_401(proxy.url);
					break;
				case 500:
					AppKit.AjaxErrorHandler.error_500(proxy.url,response);
					break;
				default:
					AppKit.AjaxErrorHandler.error_unknown(proxy.url,response);
					break;
			}
		}
		return {
			enableErrorNotifyBox : function() {
				notifyBoxEnabled = true;
			},
			disableErrorNotifyBox : function() {
				notifyBoxEnabled = false;
			},
			enableBugTrackerReport: function() {
				bugTrackerReportEnabled = true;
			},
			disableBugTrackerReport: function() {
				bugTrackerReporEnabled = false;
			},
			error_404 : function(target) {
				trackError(_("Ressource "+target+" could not be loaded - is the url correct?"))
			},
			error_500 : function(target,response) {
				var msg = 'Internal Exception, check your logs!';
				var json = {}
				try {
					json = Ext.decode(response.responseText)
					msg = (json ? json.errorMessage  : response.responseText.length <400 ? response.responseText : response.responseText.substr(0,200)+"...");
				} catch(e) {
					msg = 'Internal Exception, check your logs!';
				}
				trackError(_("The server encountered an error:<br/>")+msg,target,'XHR Request',(json ? json.isBug || false : false));
			},
			error_401 : function(target) {
				trackError(_("Access denied"));
			},
			error_unknown : function(target,error) {
				if(!(error.status)) {
					Ext.Msg.alert(_("Critical error"),_("Couldn't connect to web-server!"));
					window.location.reload();
				}
				if(!error)
					error = "Unkown error";
				trackError(_("A error occured when requesting ")+target);//+" : "+error.length <200 ? error: error.substr(0,200)+"...");
			}
		}

		
	};

	var setupErrorHandler = function() {
		var setHandlerFunc = function() {
			if(AppKit.getPreferences()["org.icinga.errorNotificationsEnabled"] == 'true') {
				AppKit.AjaxErrorHandler.enableErrorNotifyBox();
			} else {
				AppKit.AjaxErrorHandler.disableErrorNotifyBox();
			}
			if(AppKit.getPreferences()["org.icinga.bugTrackerEnabled"]  == 'true') {
				AppKit.BugTracker.setShowErrors(true);
				AppKit.AjaxErrorHandler.enableBugTrackerReport();
			} else {
				AppKit.BugTracker.setShowErrors(false);
				AppKit.AjaxErrorHandler.disableBugTrackerReport();
			}

		}
		if( AppKit.getPreferences()["org.icinga.errorNotificationsEnabled"] &&
				AppKit.getPreferences()["org.icinga.bugTrackerEnabled"]) {
			setHandlerFunc();
		} else {
			setupErrorHandler.defer(300);
		}
	}
	setupErrorHandler();



})();
