<?php

// +---------------------------------------------------------------------------+
// | This file is part of the Agavi package.                                   |
// | Copyright (c) 2005-2010 the Agavi Project.                                |
// |                                                                           |
// | For the full copyright and license information, please view the LICENSE   |
// | file that was distributed with this source code. You can also view the    |
// | LICENSE file online at http://www.agavi.org/LICENSE.txt                   |
// |   vi: set noexpandtab:                                                    |
// |   Local Variables:                                                        |
// |   indent-tabs-mode: t                                                     |
// |   End:                                                                    |
// +---------------------------------------------------------------------------+

require_once(dirname(__FILE__) . '/AgaviTask.php');

/**
 * Converts a namespaced string in dotted form to a path.
 *
 * @package    agavi
 * @subpackage build
 *
 * @author     Noah Fontes <noah.fontes@bitextender.com>
 * @copyright  Authors
 * @copyright  The Agavi Project
 *
 * @since      1.0.0
 *
 * @version    $Id: AgaviTransformnamespacestringtopathTask.php 4456 2010-03-21 19:11:45Z david $
 */
class AgaviTransformnamespacestringtopathTask extends AgaviTask
{
	protected $property = null;
	protected $string = null;

	/**
	 * Sets the property that this task will modify.
	 *
	 * @param      string The property to modify.
	 */
	public function setProperty($property)
	{
		$this->property = $property;
	}

	/**
	 * Sets the string to access for the transformation.
	 *
	 * @param      string The string to use.
	 */
	public function setString($string)
	{
		$this->string = $string;
	}

	/**
	 * Executes this target.
	 */
	public function main()
	{
		if($this->property === null) {
			throw new BuildException('The property attribute must be specified');
		}
		if($this->string === null) {
			throw new BuildException('The string attribute must be specified');
		}

		$this->project->setUserProperty($this->property, str_replace('.', '/', $this->string));
	}
}

?>